﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Text;

namespace OAI.Server.Verb {
	public class GetRecord : ServerVerb {
		private static readonly List<string> validParamNames = new List<string> { "verb", "identifier", "metadataPrefix" };

		public static string Construct(IOAIHandler oaiHandler) {
			var baseURL = oaiHandler.GetBaseURL();
			var query = oaiHandler.GetQuery();

			var identifier = query["identifier"];
			var metadataPrefix = query["metadataPrefix"];
#if DEBUG
			Debug.WriteLine(string.Format("GetRecord.Construct: identifier={0}", identifier));
			Debug.WriteLine(string.Format("GetRecord.Construct: metadataPrefix={0}", metadataPrefix));
#endif
			var sb = new StringBuilder();
			sb.Append("<?xml version=\"1.0\" encoding=\"UTF-8\" ?>");
			sb.Append("<OAI-PMH xmlns=\"http://www.openarchives.org/OAI/2.0/\"");
			sb.Append(" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"");
			sb.Append(" xsi:schemaLocation=\"http://www.openarchives.org/OAI/2.0/");
			sb.Append(" http://www.openarchives.org/OAI/2.0/OAI-PMH.xsd\">");
			sb.Append("<responseDate>");
			var buildDate = DateTime.Now;
			sb.Append(CreateResponseDate(buildDate));
			sb.Append("</responseDate>");

			sb.Append(GetRequestElement(query, validParamNames, baseURL));

			try {
				if (HasBadArguments(query, validParamNames, validParamNames)) {
					throw new BadArgumentException();
				}

				if (!oaiHandler.SupportsMetadataPrefix(metadataPrefix)) {
					throw new CannotDisseminateFormatException(metadataPrefix);
				}

				var record = oaiHandler.GetRecord(identifier, metadataPrefix, buildDate);
				if (record != null) {
					sb.Append("<GetRecord>");
					sb.Append(record);
					sb.Append("</GetRecord>");
				} else {
					throw new IdDoesNotExistException(identifier);
				}
			} catch (BadArgumentException e) {
				sb.Append(e.Message);
			} catch (CannotDisseminateFormatException e) {
				sb.Append(e.Message);
			} catch (IdDoesNotExistException e) {
				sb.Append(e.Message);
			}

			sb.Append("</OAI-PMH>");

			return sb.ToString();
		}
	}
}
